/* $Id: conf.c,v 1.140 2006/05/20 09:53:14 rav Exp $ */

/* Intro {{{
 * ----------------------------------------------------------------
 * DConnect Daemon
 *
 *	 #(@) Copyright (c) 2002, DConnect development team
 *	 #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 * ----------------------------------------------------------------
 * }}} */

#include "pch.h"

extern config_t conf;

extern int NPENAL; //
extern Tpenalty *penalties[MAXPENALTIES];

/* strdupdelim() - copy the string formed by two pointers to a new location {{{ */
char *strdupdelim(const char *beg,const char *end) {
	char *res=(char *)my_malloc(end-beg+1);

	memcpy(res,beg,end-beg);
	res[end-beg]=0;
	return res;
} /* }}} */

/* read_whole_line() - read a line from FP and return the pointer to allocated storage. {{{ */
char *read_whole_line(FILE *fp, int line_style)
{
	int length=0;
	int bufsize=82;
	char *line=(char *)my_malloc(bufsize);

	while(fgets(line+length,bufsize-length,fp))
	{
		length+=strlen(line+length);

		if (length==0) continue;
		if (line_style==LINE_NORMAL && line[length-1]=='\n') break;		
		if (line_style==LINE_PUNISHMENT && length>=2 && line[length-2]=='\004' && line[length-1]=='\n') break;
		bufsize<<=1;

		line=(char *)my_realloc(line,bufsize);
	}

	if (length==0||ferror(fp))
	{
		my_free (line);
		return NULL;
	}

	if (length+1<bufsize) line=(char *)my_realloc(line,length+1);
	return line;
} /* }}} */

/* parse_config() - parse config file {{{ */
char parse_config(int start, config_t *conf)
{
	char *line,tmp[PATH_MAX];
	FILE *fp;

	/* opening config {{{ */
	/* we look for config in location taken from configure or from command-line option */
	if (!realpath(conf->conf_main,tmp))
	{
		debug(DEBUG_SYS|DEBUG_STD,"Can't find config file (%s): %s.",conf->conf_main, strerror(errno));
		return 1;
	}

	my_duplicate(tmp,&conf->conf_main);

	fp=fopen(conf->conf_main,"rb");

	if (!fp)
	{
		debug(DEBUG_SYS|DEBUG_STD,"Can't open config file (%s): %s.",conf->conf_main, strerror(errno));
		return 1;
	}
	/* }}} */

	while((line=read_whole_line(fp,LINE_NORMAL)))
	{
		conf_cmd_exec(start,line);
		my_free(line);
	}
	
	my_free(line);
	fclose(fp);
	return 0;
} /* }}} */

/* playfile() - parse text file and play to public chat window {{{ */
void playfile(userrec_t *usr,char *msgfile)
{
	char *line;
	FILE *fp;
	char *__strtok_temp__;

	fp=fopen(msgfile,"rb");

	if (!fp)
	{
		debug(DEBUG_STD,"Can't open message file (%s): %s.",msgfile,strerror(errno));
		return;
	}

	while((line=read_whole_line(fp,LINE_NORMAL)))
	{
		if (line[0]=='\r' || line[0]=='\n') pubmsg(usr," ");
	    else 
			{
		    	strtok_r(line,"\r\n",&__strtok_temp__);
		    	pubmsg( usr, line);
			}
		
	    my_free(line);
	}

	fclose(fp);
} /* }}} */


/* validlogin() - verify login and password {{{ */
char validlogin(userrec_t *usr,char *login,char *pass)
{
	const char *sep=":\r\n";
	char *line,*p,*tmp;
	int EC=0;
	FILE *fp;

	fp=fopen(conf.conf_cusers,"rb");

	if (!fp)
	{
		debug(DEBUG_STD,"Can't open users file (%s): %s.",conf.conf_cusers,strerror(errno));
		return 0;
	}

	while((line=read_whole_line(fp,LINE_NORMAL)))
	{
		p=line;

		while(*p&&isspace(*p)) ++p;

		if (!*p||*p=='#')
		{
			my_free(line);
			continue;
		}

		tmp=strpbrk(p,sep);
		if (!tmp) goto leave;
		*(tmp++)=0;

		if (!strcasecmp(p,login))
		{		
			p=tmp;
			tmp=strpbrk(p,sep);
			if (!tmp) goto leave;
			*(tmp++)=0;

			if(usr->cons && (pass==NULL || strcmp(p,pass))) goto leave;
			else my_duplicate(p, &usr->password);

			p=tmp;
			tmp=strpbrk(p,sep);
			if (!tmp) goto leave;
			*(tmp++)=0;

			if (usr->cons) my_duplicate(p,&usr->myinfo.email);
				
			p=tmp;
			tmp=strpbrk(p,sep);
			if (!tmp) goto leave;
			*(tmp++)=0;

			if (usr->cons) my_duplicate(p,&usr->myinfo.desc);

			p=tmp;
			tmp=strpbrk(p,sep);
			if (tmp) *tmp=0;

			if (p) my_duplicate(p,&usr->perm);
			else my_duplicate("n",&usr->perm);
			
			EC=1;
			goto leave;
		}

		my_free(line);
	}

	my_duplicate("n",&usr->perm);
	
leave:
		my_free(line);
		fclose(fp);
		return EC;
} /* }}} */

/* validhost(usr, access_file, deeper){{{
.description:
	verifies host access

.args
	userrec_t *usr	userrec
	char *access_file	file name
	int deeper		if !=0 then use include: lines in the acces_file

.return values:
	0:	access file does not exists,
	host is not listed in the file

	1:	host is listed in the file
*/

char validhost(userrec_t *usr,char *access_file, int deeper)
{
	const char *sep=":\r\n";

	char *line=NULL,*p=NULL;
	char *fpath=NULL;
	FILE *fp;
	char *__strtok_temp__;

//	debug(DEBUG_PROTO,"validhost() enter");
	/* open the access_file */
	fp=fopen(access_file,"rb");

	if (!fp)
	{
		debug(DEBUG_STD,"Can't open allow/deny file (%s): %s.",access_file,strerror(errno));
		return 0;
	}

	/* parse the line */
	while((line=read_whole_line(fp,LINE_NORMAL))) /* reads the lines */
	{
		p=line; /* p contains the line */

		while(*p&&isspace(*p)) ++p;
		if (!*p||*p=='#')
		{
			my_free(line);
			continue;
		} /* skip empty and # lines; goes to the next line*/

		p=strtok_r(p,sep,&__strtok_temp__); /* read the first token of the line*/

		if (!p)
		{
			my_free(line);
			continue;
		}

		/* check for include: */
		if (deeper && !strcasecmp(p,"include"))	/* deeper!=0 */
		{
			/* parse the include:>filename< */
			p=strtok_r(NULL,sep,&__strtok_temp__); /* get the included file name */

			if (p)
			{
				my_duplicate(p,&fpath);

				make_path(CONFIGDIR,&(fpath));

				if (strcasecmp(fpath,access_file))/* file name should != NULL neither access_file */
				{
					if (validhost(usr,fpath,0))
					{
						my_free(fpath);
						goto leave;
					}//valid
				}else debug(DEBUG_STD,"Including the same file is not allowed");

				my_free(fpath);

			} else debug(DEBUG_STD,"Syntax error: include:name_of_file");

		} else/* if ip mathes one of the listed in file ips */
			if (match(p,usr->ip))
			{
				p=strtok_r(NULL,sep,&__strtok_temp__);	 /*getting reason of ban */

				if(!usr->cons)	pubmsg(usr,"%s",p?p:"Your host is banned.");

				goto leave;
			}

		my_free(line);

	}//while

	my_free(line);
	fclose(fp);

//	debug(DEBUG_PROTO,"validhost() leave");
	return 0;

leave:

	my_free(line);
	fclose(fp);
//	debug(DEBUG_PROTO,"validhost() leave");
	return 1;

} /* }}} */

/* nick_allowed() - verify nick access {{{ */
char nick_allowed(userrec_t *usr,char *nick)
{
	const char *sep=" \t\r\n";
	char *line=NULL,*p=NULL;
	FILE *fp;
	char *__strtok_temp__;

	fp=fopen(conf.conf_nallow,"rb");

	if (!fp)
	{
		debug(DEBUG_STD,"Can't open nicks allow file (%s): %s.",conf.conf_nallow,strerror(errno));
		return 1; // allow by default
	}

	while((line=read_whole_line(fp,LINE_NORMAL)))
	{
		p=line;

		while(*p&&isspace(*p)) ++p;

		if (!*p||*p=='#')
		{
			my_free(line);
			continue;
		}

		p=strtok_r(p,sep,&__strtok_temp__);
		if (!p) goto leave;

		if (match(p,nick))
		{
			my_free(line);
			fclose(fp);
			return 1;
		}

		my_free(line);
	}

leave:
	my_free(line);
	fclose(fp);
	return 0;
} /* }}} */

/* penalties_read() - read penalties list from file to memory {{{ */
void penalties_read()
{
	const char *sep=":\r\n";
	char *line=NULL,*p=NULL,*s_ip=NULL,*s_penalties, *s_start_date,*s_op,*s_nick,*s_duration,*s_reason;
	FILE *fp;
	int i=-1;
	long i_duration;
	unsigned long i_start_date;
	int i_penalties;
	char *__strtok_temp__;

//	debug(DEBUG_PROTO,"penalties_read() enter");

	fp=fopen(conf.conf_penalties,"rb");

	if (!fp)
	{
		debug(DEBUG_STD,"Can't open file (%s): %s.",conf.conf_penalties,strerror(errno));
		goto leave;
	}

	while((line=read_whole_line(fp,LINE_PUNISHMENT)))
	{
		p=line;
		i++;

		while(*p&&isspace(*p)) ++p;

		if (!*p||*p=='#')
		{
			my_free(line);
			continue;
		}

		s_ip=strtok_r(line,sep,&__strtok_temp__);
		s_nick=strtok_r(NULL,sep,&__strtok_temp__);
		s_op=strtok_r(NULL,sep,&__strtok_temp__);
		s_penalties=strtok_r(NULL,sep,&__strtok_temp__);
		s_start_date=strtok_r(NULL,sep,&__strtok_temp__);
		s_duration=strtok_r(NULL,sep,&__strtok_temp__);
		s_reason=strtok_r(NULL,"\004",&__strtok_temp__);

		if (!s_ip || !s_penalties || !s_duration || !s_reason ||
			!sscanf(s_start_date,"%lu",&i_start_date) ||
			!sscanf(s_duration,"%ld",&i_duration) ||
			!sscanf(s_penalties,"%d",&i_penalties))
		{
			debug(DEBUG_WRNG,"%s(%d): Syntax Error",conf.conf_penalties,i);
			my_free(line);
			continue;
		}

		user_set_penalty(s_ip,s_nick,s_op,i_penalties,i_start_date,i_duration,s_reason,1);

		my_free(line);
	}

leave:
//	debug(DEBUG_PROTO,"penalties_read() leave");
	my_free(line);
	fclose(fp);

} /* }}} */

/* penalties_write() - write penalties list from memory to file {{{ */
void penalties_write()
{
	FILE *fp;
	long i,duration;

//	debug(DEBUG_PROTO,"penalties_write() enter");

	fp=fopen(conf.conf_penalties,"w+");//clearing the file while opening

	if (!fp)
	{
		debug(DEBUG_STD,"Can't write penalties file (%s): %s.",conf.conf_penalties,strerror(errno));
		goto leave;
	}

	for(i=0;i<NPENAL;i++)
	{
		if(!penalties[i]) continue;
		
		duration=user_exp_penalty(penalties[i]);

	//	debug(DEBUG_PROTO,"%s %d %ld %s",penalties[i]->ip,duration,penalties[i]->penalty,penalties[i]->reason);

		if ((duration=user_exp_penalty(penalties[i]))>0) 
		{
			fprintf(fp,"%s:%s:%s:%d:%lu:%ld:%s\004\n",penalties[i]->ip,
										penalties[i]->nick,
										penalties[i]->op,
										penalties[i]->penalty,
										penalties[i]->start_date,
										duration,
										penalties[i]->reason);
			continue;
		}

		delpenalty(i);
		i--;
	}

	fclose(fp);
leave:
//	debug(DEBUG_PROTO,"penalties_write() leave");
	return;
} /* }}} */

/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */
